package Triangle;

import Triangle.AbstractSyntaxTrees.AST;
import Triangle.AbstractSyntaxTrees.ActualParameter;
import Triangle.AbstractSyntaxTrees.ActualParameterSequence;
import Triangle.AbstractSyntaxTrees.AnyTypeDenoter;
import Triangle.AbstractSyntaxTrees.ArrayAggregate;
import Triangle.AbstractSyntaxTrees.ArrayExpression;
import Triangle.AbstractSyntaxTrees.ArrayTypeDenoter;
import Triangle.AbstractSyntaxTrees.AssignCommand;
import Triangle.AbstractSyntaxTrees.BinaryExpression;
import Triangle.AbstractSyntaxTrees.BinaryOperatorDeclaration;
import Triangle.AbstractSyntaxTrees.BoolTypeDenoter;
import Triangle.AbstractSyntaxTrees.CallCommand;
import Triangle.AbstractSyntaxTrees.CallExpression;
import Triangle.AbstractSyntaxTrees.CharTypeDenoter;
import Triangle.AbstractSyntaxTrees.CharacterExpression;
import Triangle.AbstractSyntaxTrees.CharacterLiteral;
import Triangle.AbstractSyntaxTrees.Command;
import Triangle.AbstractSyntaxTrees.ConstActualParameter;
import Triangle.AbstractSyntaxTrees.ConstDeclaration;
import Triangle.AbstractSyntaxTrees.ConstFormalParameter;
import Triangle.AbstractSyntaxTrees.Declaration;
import Triangle.AbstractSyntaxTrees.DotVname;
import Triangle.AbstractSyntaxTrees.EmptyActualParameterSequence;
import Triangle.AbstractSyntaxTrees.EmptyCommand;
import Triangle.AbstractSyntaxTrees.EmptyExpression;
import Triangle.AbstractSyntaxTrees.EmptyFormalParameterSequence;
import Triangle.AbstractSyntaxTrees.ErrorTypeDenoter;
import Triangle.AbstractSyntaxTrees.Expression;
import Triangle.AbstractSyntaxTrees.FieldTypeDenoter;
import Triangle.AbstractSyntaxTrees.FormalParameter;
import Triangle.AbstractSyntaxTrees.FormalParameterSequence;
import Triangle.AbstractSyntaxTrees.FuncActualParameter;
import Triangle.AbstractSyntaxTrees.FuncDeclaration;
import Triangle.AbstractSyntaxTrees.FuncFormalParameter;
import Triangle.AbstractSyntaxTrees.Identifier;
import Triangle.AbstractSyntaxTrees.IfCommand;
import Triangle.AbstractSyntaxTrees.IfExpression;
import Triangle.AbstractSyntaxTrees.IntTypeDenoter;
import Triangle.AbstractSyntaxTrees.IntegerExpression;
import Triangle.AbstractSyntaxTrees.IntegerLiteral;
import Triangle.AbstractSyntaxTrees.LetCommand;
import Triangle.AbstractSyntaxTrees.LetExpression;
import Triangle.AbstractSyntaxTrees.MultipleActualParameterSequence;
import Triangle.AbstractSyntaxTrees.MultipleArrayAggregate;
import Triangle.AbstractSyntaxTrees.MultipleFieldTypeDenoter;
import Triangle.AbstractSyntaxTrees.MultipleFormalParameterSequence;
import Triangle.AbstractSyntaxTrees.MultipleRecordAggregate;
import Triangle.AbstractSyntaxTrees.Operator;
import Triangle.AbstractSyntaxTrees.ProcActualParameter;
import Triangle.AbstractSyntaxTrees.ProcDeclaration;
import Triangle.AbstractSyntaxTrees.ProcFormalParameter;
import Triangle.AbstractSyntaxTrees.Program;
import Triangle.AbstractSyntaxTrees.RecordAggregate;
import Triangle.AbstractSyntaxTrees.RecordExpression;
import Triangle.AbstractSyntaxTrees.RecordTypeDenoter;
import Triangle.AbstractSyntaxTrees.SequentialCommand;
import Triangle.AbstractSyntaxTrees.SequentialDeclaration;
import Triangle.AbstractSyntaxTrees.SimpleTypeDenoter;
import Triangle.AbstractSyntaxTrees.SimpleVname;
import Triangle.AbstractSyntaxTrees.SingleActualParameterSequence;
import Triangle.AbstractSyntaxTrees.SingleArrayAggregate;
import Triangle.AbstractSyntaxTrees.SingleFieldTypeDenoter;
import Triangle.AbstractSyntaxTrees.SingleFormalParameterSequence;
import Triangle.AbstractSyntaxTrees.SingleRecordAggregate;
import Triangle.AbstractSyntaxTrees.SubscriptVname;
import Triangle.AbstractSyntaxTrees.TypeDeclaration;
import Triangle.AbstractSyntaxTrees.TypeDenoter;
import Triangle.AbstractSyntaxTrees.UnaryExpression;
import Triangle.AbstractSyntaxTrees.UnaryOperatorDeclaration;
import Triangle.AbstractSyntaxTrees.VarActualParameter;
import Triangle.AbstractSyntaxTrees.VarDeclaration;
import Triangle.AbstractSyntaxTrees.VarFormalParameter;
import Triangle.AbstractSyntaxTrees.Vname;
import Triangle.AbstractSyntaxTrees.VnameExpression;
import Triangle.AbstractSyntaxTrees.WhileCommand;
import Triangle.ContextualAnalyzer.SymbolTable;
import Triangle.SyntacticAnalyzer.SourcePosition;

class ConstantFolding implements Triangle.AbstractSyntaxTrees.Visitor {

    private ErrorReporter reporter;

    @Override
    public Object visitAssignCommand(AssignCommand ast, Object o) {
        ast.E = (Expression) ast.E.visit(this, null);
        ast.V = (Vname) ast.V.visit(this, null);
        return ast;
    }

    @Override
    public Object visitCallCommand(CallCommand ast, Object o) {
        ast.APS = (ActualParameterSequence) ast.APS.visit(this, null);
        ast.I = (Identifier) ast.I.visit(this, null);
        return ast;
    }

    @Override
    public Object visitEmptyCommand(EmptyCommand ast, Object o) {
        return ast;
    }

    @Override
    public Object visitIfCommand(IfCommand ast, Object o) {
        ast.C1 = (Command) ast.C1.visit(this, null);
        ast.C2 = (Command) ast.C2.visit(this, null);
        ast.E = (Expression) ast.E.visit(this, null);
        return ast;
    }

    @Override
    public Object visitLetCommand(LetCommand ast, Object o) {
        ast.C = (Command) ast.C.visit(this, null);
        ast.D = (Declaration) ast.D.visit(this, null);
        return ast;
    }

    @Override
    public Object visitSequentialCommand(SequentialCommand ast, Object o) {
        ast.C1 = (Command) ast.C1.visit(this, null);
        ast.C2 = (Command) ast.C2.visit(this, null);
        return ast;
    }

    @Override
    public Object visitWhileCommand(WhileCommand ast, Object o) {
        ast.C = (Command) ast.C.visit(this, null);
        ast.E = (Expression) ast.E.visit(this, null);
        return ast;
    }

    @Override
    public Object visitArrayExpression(ArrayExpression ast, Object o) {
        ast.AA = (ArrayAggregate) ast.AA.visit(this, null);
        return ast;
    }

    @Override
    public Object visitBinaryExpression(BinaryExpression ast, Object o) {
        ast.E1 = (Expression) ast.E1.visit(this, null);
        ast.E2 = (Expression) ast.E2.visit(this, null);
        
        String op = ast.O.spelling;
        SourcePosition s = new SourcePosition();
        
        IntegerExpression ie0 = new IntegerExpression(new IntegerLiteral("0", s), s);
        ie0.type = StdEnvironment.integerType;
        if (isConstant(ast.E1) && isConstant(ast.E2)) {
            if (ast.E1 instanceof IntegerExpression && ast.E2 instanceof IntegerExpression) {
                String v1 = ((IntegerExpression) ast.E1).IL.spelling;
                String v2 = ((IntegerExpression) ast.E2).IL.spelling;
                int i1 = Integer.parseInt(v1);
                int i2 = Integer.parseInt(v2);
                int iRes = 0;
                boolean bRes = false, optimized = false;
                
                if (op.equals("+")) {
                    iRes = i1 + i2;
                    optimized = true;
                } else if (op.equals("-")) {
                    iRes = i1 - i2;
                    optimized = true;
                } else if (op.equals("*")) {
                    iRes = i1 * i2;
                    optimized = true;
                } else if (op.equals("/")) {
                    iRes = i1 / i2;
                    optimized = true;
                } else if (op.equals("//")) {
                    iRes = i1 % i2;
                    optimized = true;
                }
                
                System.out.println(String.format("Optimized %d %s %d to %d.", i1, op, i2, iRes));

                if (optimized) {
                    IntegerExpression i = new IntegerExpression(new IntegerLiteral(Integer.toString(iRes), s), s);
                    i.type = StdEnvironment.integerType;
                    return i; 
                }
                
                if (op.equals("<")) {
                    bRes = i1 < i2;
                    optimized = true;
                } else if (op.equals(">")) {
                    bRes = i1 > i2;
                    optimized = true;
                } else if (op.equals("<=")) {
                    bRes = i1 <= i2;
                    optimized = true;
                } else if (op.equals(">=")) {
                    bRes = i1 >= i2;
                    optimized = true;
                }
                
                if (optimized) {
                    return createBoolean(bRes);
                }
            }

            if (ast.E1 instanceof VnameExpression && ast.E2 instanceof VnameExpression) {
                boolean v1 = parseBoolean(ast.E1);
                boolean v2 = parseBoolean(ast.E2);
                boolean optimized = false;
                boolean res = false;

                if (op.equals("/\\")) {
                    res = v1 && v2;
                    optimized = true;
                } else if (op.equals("\\/")) {
                    res = v1 || v2;
                    optimized = true;
                }

                if (optimized) {
                    return createBoolean(res);
                }
            }
        } else if (isConstant(ast.E1)) {
            if (ast.E1 instanceof IntegerExpression) {
                int i = Integer.parseInt(((IntegerExpression)ast.E1).IL.spelling);
                if (i == 0) {
                    if (op.equals("+")) {
                        return ast.E2;
                    } else if (op.equals("-") && ast.E2 instanceof IntegerExpression) {
                        ((IntegerExpression)ast.E2).IL.spelling = "-"+((IntegerExpression)ast.E2).IL.spelling;
                    } else if (op.equals("*") || op.equals("/") || op.equals("//")) {
                        return ie0;
                    }
                } else if (i == 1 && op.equals("*")) {
                    return ast.E2;
                }
            } else if (ast.E1 instanceof VnameExpression) {
                boolean b = parseBoolean(ast.E1);
                
                if (op.equals("\\/")) {
                    return b? createBoolean(true) : ast.E2;
                } else if (op.equals("/\\")) {
                    return b? ast.E2 : createBoolean(false);
                }
            }
        } else if (isConstant(ast.E2)) {
            if (ast.E2 instanceof IntegerExpression) {
                int i = Integer.parseInt(((IntegerExpression)ast.E2).IL.spelling);
                if (i == 0) {
                    if (op.equals("+") || op.equals("-")) {
                        return ast.E1;
                    } else if (op.equals("*")) {
                        return ie0;
                    }
                } else if (i == 1 && op.equals("*")) {
                    return ast.E1;
                }
            } else if (ast.E2 instanceof VnameExpression) {
                boolean b = parseBoolean(ast.E2);
                
                if (op.equals("\\/")) {
                    return b? createBoolean(true) : ast.E1;
                } else if (op.equals("/\\")) {
                    return b? ast.E1 : createBoolean(false);
                }
            }
        }
        return ast;
    }

    @Override
    public Object visitCallExpression(CallExpression ast, Object o) {
        ast.APS = (ActualParameterSequence) ast.APS.visit(this, null);
        ast.I = (Identifier) ast.I.visit(this, null);
        return ast;
    }

    @Override
    public Object visitCharacterExpression(CharacterExpression ast, Object o) {
        ast.CL = (CharacterLiteral) ast.CL.visit(this, null);
        return ast;
    }

    @Override
    public Object visitEmptyExpression(EmptyExpression ast, Object o) {
        return ast;
    }

    @Override
    public Object visitIfExpression(IfExpression ast, Object o) {
        ast.E1 = (Expression) ast.E1.visit(this, null);
        ast.E2 = (Expression) ast.E2.visit(this, null);
        ast.E3 = (Expression) ast.E3.visit(this, null);
        return ast;
    }

    @Override
    public Object visitIntegerExpression(IntegerExpression ast, Object o) {
        return ast;
    }

    @Override
    public Object visitLetExpression(LetExpression ast, Object o) {
        ast.D = (Declaration) ast.D.visit(this, null);
        ast.E = (Expression) ast.E.visit(this, null);
        return ast;
    }

    @Override
    public Object visitRecordExpression(RecordExpression ast, Object o) {
        ast.RA = (RecordAggregate) ast.RA.visit(this, null);
        return ast;
    }

    @Override
    public Object visitUnaryExpression(UnaryExpression ast, Object o) {
        ast.E = (Expression) ast.E.visit(this, null);
        if (isConstant(ast.E) && ast.E instanceof VnameExpression) {
            boolean b = parseBoolean(ast.E);
            if (ast.O.spelling.equals("\\")) {
                return createBoolean(!b);
            }
        }        
        return ast;
    }

    @Override
    public Object visitVnameExpression(VnameExpression ast, Object o) {
        ast.V = (Vname) ast.V.visit(this, null);
        return ast;
    }

    @Override
    public Object visitBinaryOperatorDeclaration(BinaryOperatorDeclaration ast, Object o) {
        ast.ARG1 = (TypeDenoter) ast.ARG1.visit(this, null);
        ast.ARG2 = (TypeDenoter) ast.ARG2.visit(this, null);
        ast.O = (Operator) ast.O.visit(this, null);
        ast.RES = (TypeDenoter) ast.RES.visit(this, null);
        return ast;
    }

    @Override
    public Object visitConstDeclaration(ConstDeclaration ast, Object o) {
        ast.E = (Expression) ast.E.visit(this, null);
        ast.I = (Identifier) ast.I.visit(this, null);
        return ast;
    }

    @Override
    public Object visitFuncDeclaration(FuncDeclaration ast, Object o) {
        ast.E = (Expression) ast.E.visit(this, null);
        ast.FPS = (FormalParameterSequence) ast.FPS.visit(this, null);
        ast.I = (Identifier) ast.I.visit(this, null);
        ast.T = (TypeDenoter) ast.T.visit(this, null);
        return ast;
    }

    @Override
    public Object visitProcDeclaration(ProcDeclaration ast, Object o) {
        ast.C = (Command) ast.C.visit(this, null);
        ast.FPS = (FormalParameterSequence) ast.FPS.visit(this, null);
        ast.I = (Identifier) ast.I.visit(this, null);
        return ast;
    }

    @Override
    public Object visitSequentialDeclaration(SequentialDeclaration ast, Object o) {
        ast.D1 = (Declaration) ast.D1.visit(this, null);
        ast.D2 = (Declaration) ast.D2.visit(this, null);
        return ast;
    }

    @Override
    public Object visitTypeDeclaration(TypeDeclaration ast, Object o) {
        ast.I = (Identifier) ast.I.visit(this, null);
        ast.T = (TypeDenoter) ast.T.visit(this, null);
        return ast;
    }

    @Override
    public Object visitUnaryOperatorDeclaration(UnaryOperatorDeclaration ast, Object o) {
        ast.ARG = (TypeDenoter) ast.ARG.visit(this, null);
        ast.O = (Operator) ast.O.visit(this, null);
        ast.RES = (TypeDenoter) ast.RES.visit(this, null);
        return ast;
    }

    @Override
    public Object visitVarDeclaration(VarDeclaration ast, Object o) {
        ast.I = (Identifier) ast.I.visit(this, null);
        ast.T = (TypeDenoter) ast.T.visit(this, null);
        return ast;
    }

    @Override
    public Object visitMultipleArrayAggregate(MultipleArrayAggregate ast, Object o) {
        ast.AA = (ArrayAggregate) ast.AA.visit(this, null);
        ast.E = (Expression) ast.E.visit(this, null);
        return ast;
    }

    @Override
    public Object visitSingleArrayAggregate(SingleArrayAggregate ast, Object o) {
        ast.E = (Expression) ast.E.visit(this, null);
        return ast;
    }

    @Override
    public Object visitMultipleRecordAggregate(MultipleRecordAggregate ast, Object o) {
        ast.E = (Expression) ast.E.visit(this, null);
        ast.I = (Identifier) ast.I.visit(this, null);
        ast.RA = (RecordAggregate) ast.RA.visit(this, null);
        return ast;
    }

    @Override
    public Object visitSingleRecordAggregate(SingleRecordAggregate ast, Object o) {
        ast.E = (Expression) ast.E.visit(this, null);
        ast.I = (Identifier) ast.I.visit(this, null);
        return ast;
    }

    @Override
    public Object visitConstFormalParameter(ConstFormalParameter ast, Object o) {
        ast.I = (Identifier) ast.I.visit(this, null);
        ast.T = (TypeDenoter) ast.T.visit(this, null);
        return ast;
    }

    @Override
    public Object visitFuncFormalParameter(FuncFormalParameter ast, Object o) {
        ast.FPS = (FormalParameterSequence) ast.FPS.visit(this, null);
        ast.I = (Identifier) ast.I.visit(this, null);
        ast.T = (TypeDenoter) ast.T.visit(this, null);
        return ast;
    }

    @Override
    public Object visitProcFormalParameter(ProcFormalParameter ast, Object o) {
        ast.FPS = (FormalParameterSequence) ast.FPS.visit(this, null);
        ast.I = (Identifier) ast.I.visit(this, null);
        return ast;
    }

    @Override
    public Object visitVarFormalParameter(VarFormalParameter ast, Object o) {
        ast.I = (Identifier) ast.I.visit(this, null);
        ast.T = (TypeDenoter) ast.T.visit(this, null);
        return ast;
    }

    @Override
    public Object visitEmptyFormalParameterSequence(EmptyFormalParameterSequence ast, Object o) {
        return ast;
    }

    @Override
    public Object visitMultipleFormalParameterSequence(MultipleFormalParameterSequence ast, Object o) {
        ast.FP = (FormalParameter) ast.FP.visit(this, null);
        ast.FPS = (FormalParameterSequence) ast.FPS.visit(this, null);
        return ast;
    }

    @Override
    public Object visitSingleFormalParameterSequence(SingleFormalParameterSequence ast, Object o) {
        ast.FP = (FormalParameter) ast.FP.visit(this, null);
        return ast;
    }

    @Override
    public Object visitConstActualParameter(ConstActualParameter ast, Object o) {
        ast.E = (Expression) ast.E.visit(this, null);
        return ast;
    }

    @Override
    public Object visitFuncActualParameter(FuncActualParameter ast, Object o) {
        ast.I = (Identifier) ast.I.visit(this, null);
        return ast;
    }

    @Override
    public Object visitProcActualParameter(ProcActualParameter ast, Object o) {
        ast.I = (Identifier) ast.I.visit(this, null);
        return ast;
    }

    @Override
    public Object visitVarActualParameter(VarActualParameter ast, Object o) {
        ast.V = (Vname) ast.V.visit(this, null);
        return ast;
    }

    @Override
    public Object visitEmptyActualParameterSequence(EmptyActualParameterSequence ast, Object o) {
        return ast;
    }

    @Override
    public Object visitMultipleActualParameterSequence(MultipleActualParameterSequence ast, Object o) {
        ast.AP = (ActualParameter) ast.AP.visit(this, null);
        ast.APS = (ActualParameterSequence) ast.APS.visit(this, null);
        return ast;
    }

  @Override
    public Object visitSingleActualParameterSequence(SingleActualParameterSequence ast, Object o) {
        ast.AP = (ActualParameter) ast.AP.visit(this, null);
        return ast;
    }

    @Override
    public Object visitAnyTypeDenoter(AnyTypeDenoter ast, Object o) {
        return ast;
    }

    @Override
    public Object visitArrayTypeDenoter(ArrayTypeDenoter ast, Object o) {
        ast.IL = (IntegerLiteral) ast.IL.visit(this, null);
        ast.T = (TypeDenoter) ast.T.visit(this, null);
        return ast;
    }

    @Override
    public Object visitBoolTypeDenoter(BoolTypeDenoter ast, Object o) {
        return ast;
    }

    @Override
    public Object visitCharTypeDenoter(CharTypeDenoter ast, Object o) {
        return ast;
    }

    @Override
    public Object visitErrorTypeDenoter(ErrorTypeDenoter ast, Object o) {
        return ast;
    }

    @Override
    public Object visitSimpleTypeDenoter(SimpleTypeDenoter ast, Object o) {
        ast.I = (Identifier) ast.I.visit(this, null);
        return ast;
    }

    @Override
    public Object visitIntTypeDenoter(IntTypeDenoter ast, Object o) {
        return ast;
    }

    @Override
    public Object visitRecordTypeDenoter(RecordTypeDenoter ast, Object o) {
        ast.FT = (FieldTypeDenoter) ast.FT.visit(this, null);
        return ast;
    }

    @Override
    public Object visitMultipleFieldTypeDenoter(MultipleFieldTypeDenoter ast, Object o) {
        ast.FT = (FieldTypeDenoter) ast.FT.visit(this, null);
        ast.T = (TypeDenoter) ast.T.visit(this, null);
        ast.I = (Identifier) ast.I.visit(this, null);
        return ast;
    }

    @Override
    public Object visitSingleFieldTypeDenoter(SingleFieldTypeDenoter ast, Object o) {
        ast.I = (Identifier) ast.I.visit(this, null);
        ast.T = (TypeDenoter) ast.T.visit(this, null);
        return ast;
    }

    @Override
    public Object visitCharacterLiteral(CharacterLiteral ast, Object o) {
        return ast;
    }

    @Override
    public Object visitIdentifier(Identifier ast, Object o) {
        return ast;
    }

    @Override
    public Object visitIntegerLiteral(IntegerLiteral ast, Object o) {
        return ast;
    }

    @Override
    public Object visitOperator(Operator ast, Object o) {
        return ast;
    }

    @Override
    public Object visitDotVname(DotVname ast, Object o) {
        ast.V = (Vname) ast.V.visit(this, null);
        return ast;
    }

    @Override
    public Object visitSimpleVname(SimpleVname ast, Object o) {
        return ast;
    }

    @Override
    public Object visitSubscriptVname(SubscriptVname ast, Object o) {
        ast.E = (Expression) ast.E.visit(this, null);
        ast.V = (Vname) ast.V.visit(this, null);
        return ast;
    }

    @Override
    public Object visitProgram(Program ast, Object o) {
        ast.C = (Command) ast.C.visit(this, null);
        return ast;
    }

    public void optimize(Program ast) {
        ast.visit(this, null);
    }
    
    public ConstantFolding(ErrorReporter reporter) {
        this.reporter = reporter;
    }
    
    /**
     * Returns <code>true</code> if the given expression represents constant value.
     * This value can be a boolean or integer.
     * @param e the expression to test
     * @return <code>true</code> if the given expression represents constant value
     */
    private boolean isConstant(Expression e) {
        if (e instanceof IntegerExpression) {
            return true;
        }
        if (e instanceof VnameExpression) {
            VnameExpression vname = (VnameExpression) e;
            if (vname.V instanceof SimpleVname) {
                if (((SimpleVname) vname.V).I.spelling.equals(StdEnvironment.trueDecl.I.spelling)
                        || ((SimpleVname) vname.V).I.spelling.equals(StdEnvironment.falseDecl.I.spelling)) {
                    return true;
                }
            }
        }
        return false;
    }

    /**
     * Returns the boolean value of the given expression that represents a boolean value.
     * @param e a boolean value
     * @return the boolean value of the given expression that represents a boolean value
     */
    private boolean parseBoolean(Expression e) {
        if (e instanceof VnameExpression) {
            VnameExpression vname = (VnameExpression) e;
            if (vname.V instanceof SimpleVname) {
                if (((SimpleVname) vname.V).I.spelling.equals(StdEnvironment.trueDecl.I.spelling)) {
                    return true;
                }
            }
            if (((SimpleVname) vname.V).I.spelling.equals(StdEnvironment.falseDecl.I.spelling)) {
                return false;
            }
        }
        throw new RuntimeException();
    }

    /**
     * Returns an expression representing the given boolean value.
     * @param val the boolean value that will be represented by the returned expression
     * @return an expression representing the given boolean value
     */
    private Expression createBoolean(boolean val) {
        Identifier iAST = new Identifier(val ? "true" : "false", new SourcePosition());
        iAST.decl = val ? StdEnvironment.trueDecl : StdEnvironment.falseDecl;
        SimpleVname vAST = new SimpleVname(iAST, new SourcePosition());
        vAST.type = StdEnvironment.booleanType;
        VnameExpression boolExpression = new VnameExpression(vAST, new SourcePosition());
        boolExpression.type = StdEnvironment.booleanType;
        return boolExpression;
    }
}
