package Triangle.ContextualAnalyzer;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Stack;

import Triangle.AbstractSyntaxTrees.Declaration;

public final class SymbolTable {

	private Map<String, Stack<Declaration>> symtab;
	private Stack<List<String>> scopeStack;

	public SymbolTable() {
		symtab = new HashMap<String, Stack<Declaration>>();
		scopeStack = new Stack<List<String>>();
		// For Global symbols:
		scopeStack.push(new ArrayList<String>());
	}

	// Opens a new level in the identification table, 1 higher than the
	// current topmost level.
	public void openScope() {
		List<String> list = new ArrayList<String>();
		scopeStack.push(list);
	}

	// Closes the topmost level in the identification table, discarding
	// all entries belonging to that level.
	public void closeScope() {
		List<String> list = scopeStack.pop();
		for (String id : list) {
			if (symtab.get(id).size() == 1)
				symtab.remove(id);
			else
				symtab.get(id).pop();
		}
	}

	// Makes a new entry in the identification table for the given identifier
	// and attribute. The new entry belongs to the current level.
	// duplicated is set to to true if there is already an entry for the
	// same identifier at the current level.
	public void enter(String id, Declaration attr) {
		List<String> list = scopeStack.lastElement();
		list.add(id);
		if (symtab.containsKey(id))
			symtab.get(id).add(attr);
		else {
			Stack<Declaration> stack = new Stack<Declaration>();
			stack.add(attr);
			symtab.put(id, stack);
		}
	}

	// Finds an entry for the given identifier in the identification table,
	// if any. If there are several entries for that identifier, finds the
	// entry at the highest level, in accordance with the scope rules.
	// Returns null if no entry is found.
	// otherwise returns the attribute field of the entry found.
	public Declaration retrieve(String id) {
		if (!symtab.containsKey(id))
			return null;
		return symtab.get(id).lastElement();
	}

}
